@@ -23,7 +23,7 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
-__VERSION__ = "2.0.6"
+__VERSION__ = "2.0.9"
 
 import urllib, urllib2, base64, hmac
 from hashlib import sha1
@@ -66,16 +66,16 @@ class Account:
     """
     def __init__(self, id, token):
         """initialize a twilio account object
-        
+
         id: Twilio account SID/ID
         token: Twilio account token
-        
+
         returns a Twilio account object
         """
         self.id = id
         self.token = token
         self.opener = None
-    
+
     def _build_get_uri(self, uri, params):
         if params and len(params) > 0:
             if uri.find('?') > 0:
@@ -85,13 +85,13 @@ class Account:
             else:
                 uri = uri + '?' + urllib.urlencode(params)
         return uri
-    
+
     def _urllib2_fetch(self, uri, params, method=None):
         # install error processor to handle HTTP 201 response correctly
         if self.opener == None:
             self.opener = urllib2.build_opener(HTTPErrorProcessor)
             urllib2.install_opener(self.opener)
-        
+
         if method and method == 'GET':
             uri = self._build_get_uri(uri, params)
             req = TwilioUrlRequest(uri)
@@ -99,24 +99,24 @@ class Account:
             req = TwilioUrlRequest(uri, urllib.urlencode(params))
             if method and (method == 'DELETE' or method == 'PUT'):
                 req.http_method = method
-        
+
         authstring = base64.encodestring('%s:%s' % (self.id, self.token))
         authstring = authstring.replace('\n', '')
         req.add_header("Authorization", "Basic %s" % authstring)
-        
+
         response = urllib2.urlopen(req)
         return response.read()
-    
+
     def _appengine_fetch(self, uri, params, method):
         if method == 'GET':
             uri = self._build_get_uri(uri, params)
-        
+
         try:
             httpmethod = getattr(urlfetch, method)
         except AttributeError:
             raise NotImplementedError(
                 "Google App Engine does not support method '%s'" % method)
-        
+
         authstring = base64.encodestring('%s:%s' % (self.id, self.token))
         authstring = authstring.replace('\n', '')
         r = urlfetch.fetch(url=uri, payload=urllib.urlencode(params),
@@ -127,14 +127,14 @@ class Account:
             raise HTTPErrorAppEngine("HTTP %s: %s" % \
                 (r.status_code, r.content))
         return r.content
-    
+
     def request(self, path, method=None, vars={}):
         """sends a request and gets a response from the Twilio REST API
-        
+
         path: the URL (relative to the endpoint URL, after the /v1
         url: the HTTP method to use, defaults to POST
         vars: for POST or PUT, a dict of data to send
-        
+
         returns Twilio response in XML or raises an exception on error
         """
         if not path or len(path) < 1:
@@ -142,12 +142,12 @@ class Account:
         if method and method not in ['GET', 'POST', 'DELETE', 'PUT']:
             raise NotImplementedError(
                 'HTTP %s method not implemented' % method)
-        
+
         if path[0] == '/':
             uri = _TWILIO_API_URL + path
         else:
             uri = _TWILIO_API_URL + '/' + path
-        
+
         if APPENGINE:
             return self._appengine_fetch(uri, vars, method)
         return self._urllib2_fetch(uri, vars, method)
@@ -162,13 +162,13 @@ class Verb:
         self.name = self.__class__.__name__
         self.body = None
         self.nestables = None
-        
+
         self.verbs = []
         self.attrs = {}
         for k, v in kwargs.items():
             if k == "sender": k = "from"
-            if v: self.attrs[k] = quoteattr(str(v))
-    
+            if v != None: self.attrs[k] = quoteattr(str(v))
+
     def __repr__(self):
         s = '<%s' % self.name
         keys = self.attrs.keys()
@@ -188,7 +188,7 @@ class Verb:
         else:
             s += '/>\n'
         return s
-    
+
     def append(self, verb):
         if not self.nestables:
             raise TwilioException("%s is not nestable" % self.name)
@@ -197,56 +197,59 @@ class Verb:
                 (verb.name, self.name))
         self.verbs.append(verb)
         return verb
-    
+
     def asUrl(self):
         return urllib.quote(str(self))
-    
+
     def addSay(self, text, **kwargs):
         return self.append(Say(text, **kwargs))
-    
+
     def addPlay(self, url, **kwargs):
         return self.append(Play(url, **kwargs))
-    
+
     def addPause(self, **kwargs):
         return self.append(Pause(**kwargs))
-    
+
     def addRedirect(self, url=None, **kwargs):
-        return self.append(Redirect(url, **kwargs))   
-    
+        return self.append(Redirect(url, **kwargs))
+
     def addHangup(self, **kwargs):
-        return self.append(Hangup(**kwargs)) 
-    
+        return self.append(Hangup(**kwargs))
+
+    def addReject(self, **kwargs):
+        return self.append(Reject(**kwargs))
+
     def addGather(self, **kwargs):
         return self.append(Gather(**kwargs))
-    
+
     def addNumber(self, number, **kwargs):
         return self.append(Number(number, **kwargs))
-    
+
     def addDial(self, number=None, **kwargs):
         return self.append(Dial(number, **kwargs))
-    
+
     def addRecord(self, **kwargs):
         return self.append(Record(**kwargs))
-    
+
     def addConference(self, name, **kwargs):
         return self.append(Conference(name, **kwargs))
-        
+
     def addSms(self, msg, **kwargs):
         return self.append(Sms(msg, **kwargs))
 
 class Response(Verb):
     """Twilio response object.
-    
+
     version: Twilio API version e.g. 2008-08-01
     """
     def __init__(self, version=None, **kwargs):
         Verb.__init__(self, version=version, **kwargs)
         self.nestables = ['Say', 'Play', 'Gather', 'Record', 'Dial',
-            'Redirect', 'Pause', 'Hangup', 'Sms']
+            'Redirect', 'Pause', 'Hangup', 'Reject', 'Sms']
 
 class Say(Verb):
     """Say text
-    
+
     text: text to say
     voice: MAN or WOMAN
     language: language to use
@@ -254,12 +257,12 @@ class Say(Verb):
     """
     MAN = 'man'
     WOMAN = 'woman'
-    
+
     ENGLISH = 'en'
     SPANISH = 'es'
     FRENCH = 'fr'
     GERMAN = 'de'
-    
+
     def __init__(self, text, voice=None, language=None, loop=None, **kwargs):
         Verb.__init__(self, voice=voice, language=language, loop=loop,
             **kwargs)
@@ -267,14 +270,15 @@ class Say(Verb):
         if voice and (voice != self.MAN and voice != self.WOMAN):
             raise TwilioException( \
                 "Invalid Say voice parameter, must be 'man' or 'woman'")
-        if voice and (voice != self.MAN and voice != self.WOMAN):
+        if language and (language != self.ENGLISH and language != self.SPANISH
+            and language != self.FRENCH and language != self.GERMAN):
             raise TwilioException( \
                 "Invalid Say language parameter, must be " + \
                 "'en', 'es', 'fr', or 'de'")
 
 class Play(Verb):
     """Play audio file at a URL
-    
+
     url: url of audio file, MIME type on file must be set correctly
     loop: number of time to say this text
     """
@@ -284,7 +288,7 @@ class Play(Verb):
 
 class Pause(Verb):
     """Pause the call
-    
+
     length: length of pause in seconds
     """
     def __init__(self, length=None, **kwargs):
@@ -292,12 +296,12 @@ class Pause(Verb):
 
 class Redirect(Verb):
     """Redirect call flow to another URL
-    
+
     url: redirect url
     """
     GET = 'GET'
     POST = 'POST'
-    
+
     def __init__(self, url=None, method=None, **kwargs):
         Verb.__init__(self, method=method, **kwargs)
         if method and (method != self.GET and method != self.POST):
@@ -313,7 +317,7 @@ class Hangup(Verb):
 
 class Gather(Verb):
     """Gather digits from the caller's keypad
-    
+
     action: URL to which the digits entered will be sent
     method: submit to 'action' url using GET or POST
     numDigits: how many digits to gather before returning
@@ -325,7 +329,7 @@ class Gather(Verb):
 
     def __init__(self, action=None, method=None, numDigits=None, timeout=None,
         finishOnKey=None, **kwargs):
-        
+
         Verb.__init__(self, action=action, method=method,
             numDigits=numDigits, timeout=timeout, finishOnKey=finishOnKey,
             **kwargs)
@@ -336,17 +340,17 @@ class Gather(Verb):
 
 class Number(Verb):
     """Specify phone number in a nested Dial element.
-    
+
     number: phone number to dial
     sendDigits: key to press after connecting to the number
     """
     def __init__(self, number, sendDigits=None, **kwargs):
         Verb.__init__(self, sendDigits=sendDigits, **kwargs)
         self.body = number
-        
+
 class Sms(Verb):
     """ Send a Sms Message to a phone number
-    
+
     to: whom to send message to, defaults based on the direction of the call
     sender: whom to send message from.
     action: url to request after the message is queued
@@ -355,7 +359,7 @@ class Sms(Verb):
     """
     GET = 'GET'
     POST = 'POST'
-    
+
     def __init__(self, msg, to=None, sender=None, method=None, action=None,
         statusCallback=None, **kwargs):
         Verb.__init__(self, action=action, method=method, to=to, sender=sender,
@@ -367,8 +371,8 @@ class Sms(Verb):
 
 class Conference(Verb):
     """Specify conference in a nested Dial element.
-    
-    name: friendly name of conference 
+
+    name: friendly name of conference
     muted: keep this participant muted (bool)
     beep: play a beep when this participant enters/leaves (bool)
     startConferenceOnEnter: start conf when this participants joins (bool)
@@ -378,7 +382,7 @@ class Conference(Verb):
     """
     GET = 'GET'
     POST = 'POST'
-    
+
     def __init__(self, name, muted=None, beep=None,
         startConferenceOnEnter=None, endConferenceOnExit=None, waitUrl=None,
         waitMethod=None, **kwargs):
@@ -393,13 +397,13 @@ class Conference(Verb):
 
 class Dial(Verb):
     """Dial another phone number and connect it to this call
-    
+
     action: submit the result of the dial to this URL
     method: submit to 'action' url using GET or POST
     """
     GET = 'GET'
     POST = 'POST'
-    
+
     def __init__(self, number=None, action=None, method=None, **kwargs):
         Verb.__init__(self, action=action, method=method, **kwargs)
         self.nestables = ['Number', 'Conference']
@@ -414,7 +418,7 @@ class Dial(Verb):
 
 class Record(Verb):
     """Record audio from caller
-    
+
     action: submit the result of the dial to this URL
     method: submit to 'action' url using GET or POST
     maxLength: maximum number of seconds to record
@@ -422,46 +426,60 @@ class Record(Verb):
     """
     GET = 'GET'
     POST = 'POST'
-    
-    def __init__(self, action=None, method=None, maxLength=None, 
-        timeout=None, **kwargs):
+
+    def __init__(self, action=None, method=None, maxLength=None,
+                 timeout=None, **kwargs):
         Verb.__init__(self, action=action, method=method, maxLength=maxLength,
             timeout=timeout, **kwargs)
         if method and (method != self.GET and method != self.POST):
             raise TwilioException( \
                 "Invalid method parameter, must be GET or POST")
 
+class Reject(Verb):
+    """Reject an incoming call
+
+    reason: message to play when rejecting a call
+    """
+    REJECTED = 'rejected'
+    BUSY = 'busy'
+
+    def __init__(self, reason=None, **kwargs):
+        Verb.__init__(self, reason=reason, **kwargs)
+        if reason and (reason != self.REJECTED and reason != self.BUSY):
+            raise TwilioException( \
+                "Invalid reason parameter, must be BUSY or REJECTED")
+
 # Twilio Utility function and Request Validation
 # ===========================================================================
 
 class Utils:
     def __init__(self, id, token):
         """initialize a twilio utility object
-        
+
         id: Twilio account SID/ID
         token: Twilio account token
-        
+
         returns a Twilio util object
         """
         self.id = id
         self.token = token
-    
+
     def validateRequest(self, uri, postVars, expectedSignature):
         """validate a request from twilio
-        
+
         uri: the full URI that Twilio requested on your server
         postVars: post vars that Twilio sent with the request
         expectedSignature: signature in HTTP X-Twilio-Signature header
-        
+
         returns true if the request passes validation, false if not
         """
-        
+
         # append the POST variables sorted by key to the uri
         s = uri
         if len(postVars) > 0:
             for k, v in sorted(postVars.items()):
                 s += k + v
-        
+
         # compute signature and compare signatures
         return (base64.encodestring(hmac.new(self.token, s, sha1).digest()).\
             strip() == expectedSignature)
